/* @font-face {
  font-family: "Quicksand";
  src: url("fonts/Quicksand.ttf") format("truetype");
} */

@font-face {
  font-family: "Inter";
  src: url("fonts/Inter.ttf") format("truetype");
}

/* LANDING TAB */
#landing-section {
  display: flex;
  flex-direction: column;
  margin-left: auto;
  margin-right: auto;
}

[title] {
  text-decoration-line: underline;
  text-decoration-style: dotted;
  text-decoration-color: var(--bs-primary);
  text-decoration-thickness: 0.1rem;
  color: var(--bs-primary);
  cursor: help;
}

.tooltip-underline {
  /* text-decoration: underline #888 dotted; */
  cursor: help;
}

h1 {
  padding-bottom: 1rem;
}
