/* HEADER */

h1 {
  font-weight: black;
  margin: 0 2rem 0 0;
}

.material-symbols-rounded {
  font-variation-settings: "FILL" 0, "wght" 400, "GRAD" 0, "opsz" 24;
}

.title-section {
  display: flex;
  flex-direction: row;
  padding-top: 1rem;
}

.svg-icons {
  width: 26px;
  height: 26px;
  margin: 3px 0;
}

/* FOOTER */
.footer-p {
  font-size: 12px;
}
