.figure-title {
  font-weight: bold;
}

#figure-container {
  display: flex;
  flex-wrap: wrap;
}

.figure-card {
  padding: 30px 15px 30px 15px;
  margin: 0 15px 15px 0;
  min-height: 100px;
  display: flex;
  width: 100%;
  border-radius: 16px;
}

.figure-description {
  margin-bottom: 2rem;
}
